	INCLUDE EFR32MG12.inc			; CPU register definitions
	INCLUDE	i2c.inc	
	IMPORT	I2Cbuf	

SI1133_SLAVE_ADDRESS	EQU	0x55    	; sensor slave address

;-------------------------------------	; non-initialized DATA segment in RAM 
	AREA	RAM, DATA, NOINIT, READWRITE, ALIGN=3
ch0data	SPACE	2	
	EXPORT	ch0data				
		
	AREA    |.text|, CODE, READONLY		; another section in the CODE area	
;-------------------------------------------------------------------------------
SI1133_setup	PROC
	EXPORT	SI1133_setup
	push	{LR, R0-R1}
	ldr	R0, =10000000
ss	subs	R0, #1				; power-up delay
	bne	ss	
	
	PUTB	0x01, 1, I2Cbuf			; CHAIN_LIST data (enable ch0)
	PUTB	0x81, 2				; PARAM_SET (CHAIN_LIST address 0x01)
	bl	SI1133_set_parameter

	PUTB	0x0D, 1, I2Cbuf			; ADCCONFIG0 data (large white)
	PUTB	0x82, 2				; PARAM_SET (ADCCONFIG0 address 0x02)
	bl	SI1133_set_parameter

	PUTB	0x33, 1, I2Cbuf			; ADCSENS0 data (SW_GAIN=8, HW_GAIN=8)
	PUTB	0x83, 2				; PARAM_SET (ADCSENS0 address 0x03)
	bl	SI1133_set_parameter

	PUTB	0x18, 1, I2Cbuf			; ADCPOST0 data (POSTSHIFT=3)
	PUTB	0x84, 2				; PARAM_SET (ADCPOST0 address 0x04)
	bl	SI1133_set_parameter

;	PUTB	0x0F, 0, I2Cbuf			; IRQ_ENABLE reg address
;	PUTB	0x01, 1				; enable IRQ from ch0
;	mov	R0, #SI1133_SLAVE_ADDRESS
;	ldr	R1, =I2Cbuf
;	mov	R2, #2				; write 2 bytes
;	bl	I2CMasterWriteBuf
	pop	{PC, R0-R1}	
	ENDP

;-------------------------------------------------------------------------------
SI1133_set_parameter	PROC
	push	{LR, R0-R3}
	bl	SI1133_read_response
	and	R3, #0x0F			; leave only CMMND_CTR bits
	push	{R3}				; save response
	mov	R0, #SI1133_SLAVE_ADDRESS
	ldr	R1, =I2Cbuf
	mov	R2, #0x0A			; INPUT0 reg address
	strb	R2, [R1]			; load reg address 
	mov	R2, #3				; write 3 bytes from I2Cbuf
	bl	I2CMasterWriteBuf
	pop	{R2}
rr	bl	SI1133_read_response		; wait for command to complete
	and	R3, #0x0F
	eors	R3, R2, R3
	beq	rr
	pop	{PC, R0-R3}	
	ENDP

;-------------------------------------------------------------------------------
SI1133_send_command	PROC
	push	{LR, R0-R3}
	bl	SI1133_read_response
	and	R3, #0x0F			; leave only CMMND_CTR bits
	push	{R3}				; save response
	mov	R0, #SI1133_SLAVE_ADDRESS
	ldr	R1, =I2Cbuf
	mov	R2, #0x0B			; COMMAND reg address
	strb	R2, [R1]			; load reg address 
	mov	R2, #2				; write 2 bytes from I2Cbuf
	bl	I2CMasterWriteBuf
	pop	{R2}
sc	bl	SI1133_read_response		; wait for command to complete
	and	R3, #0x0F
	eors	R3, R2, R3
	beq	sc
	pop	{PC, R0-R3}
	ENDP

;-------------------------------------------------------------------------------
SI1133_read_response	PROC
	push	{LR, R0-R2}
	mov	R0, #SI1133_SLAVE_ADDRESS
	ldr	R1, =I2Cbuf
	mov	R2, #1				; read 1 byte
	mov	R3, #0x11			; RESPONSE0 reg address
	bl	I2CMasterReadBuf
	ldrb	R3, [R1, #0]			; R3 = RESPONSE0
	pop	{PC, R0-R2}		
	ENDP
		
;-------------------------------------------------------------------------------
SI1133_request_conversion	PROC
	EXPORT	SI1133_request_conversion
	push	{LR, R0-R1}			; save used regs	
	ldr	R0, =I2Cbuf
	mov	R1, #0x11			; FORCE command
	strb	R1, [R0, #1]
	bl	SI1133_send_command
	pop	{PC, R0-R1}
	ENDP
		
;-------------------------------------------------------------------------------
SI1133_get_data	PROC
	EXPORT	SI1133_get_data
	push	{LR, R0-R3}			; save used regs
	mov	R0, #SI1133_SLAVE_ADDRESS
	ldr	R1, =I2Cbuf
	mov	R2, #2				; read 2 bytes
	mov	R3, #0x13			; from HOSTOUT0 reg addrses
	bl	I2CMasterReadBuf

	ldrh	R2, [R1]			; get ch0 data
	rev16	R2, R2				; reverse bytes
	ldr	R0, =ch0data
	strh	R2, [R0]			; save it in RAM
	pop	{PC, R0-R3}
	ENDP

	ALIGN
	END	